from tuprolog import Info
from tuprolog.core import *
from tuprolog.core.comparators import *
from tuprolog.core.exception import *
from tuprolog.core.formatters import *
from tuprolog.core.operators import *
from tuprolog.core.visitors import *
from tuprolog.core.parsing import *
from tuprolog.unify import *
from tuprolog.theory import *
from tuprolog.theory.parsing import *
from tuprolog.solve import *

import code

code.interact(local=locals())



import logging
import jpype
import jpype.imports

from it.unibo.tuprolog.core.exception import TuPrologException
from it.unibo.tuprolog.core.exception import SubstitutionException
from it.unibo.tuprolog.core.exception import SubstitutionApplicationException

logging.debug("Loaded JVM classes from it.unibo.tuprolog.core.exception.*")




import logging
import jpype
import jpype.imports

from io import IOBase, TextIOBase, SEEK_SET
from java.io import InputStream, Reader, InputStreamReader, BufferedReader
from java.nio.file import Files, Paths

from typing import Union

@jpype.JImplementationFor("java.io.InputStream")
class _JvmInputStream:
    def __jclass_init__(cls):
        IOBase.register(cls)

    @jpype.JOverride
    def close(self):
        self._closed = True
        self.close_()

    @property
    def closed(self):
        if hasattr(self, '_closed'):
            return self._closed
        else:
            return False

    def fileno(self):
        raise OSError("This IOBase is backed by an instance of <java.io.InputStream>")

    def flush(self):
        pass

    def isatty(self):
        return False

    def readable(self):
        return True

    def seekable(self): 
        return False

    def writable(self): 
        return False

    def _buffer(self):
        if not hasattr(self, '_buffered'):
            self._buffered = BufferedReader(InputStreamReader(self))

    def readline(self, size=-1):
        self._buffer()
        self._buffered.readLine()

    def readlines(self, hint=-1):
        self._buffer()
        stream = self._buffered.read.lines()
        if hint >= 0:
            stream = stream.limit(hint)
        return list(stream)

    def seek(offset: int, whence=SEEK_SET):
        raise OSError("This IOBase is backed by an instance of <java.io.InputStream>")

    def tell():
        raise OSError("This IOBase is backed by an instance of <java.io.InputStream>")

    def truncate(size=None):
        raise OSError("This IOBase is backed by an instance of <java.io.InputStream>")

    def writelines(self):
        raise OSError("This IOBase is backed by an instance of <java.io.InputStream>")


def open_file(path: str) -> InputStream:
    return Files.newInputStream(Paths.get(path))


def ensure_input_steam(input: Union[str, InputStream]) -> InputStream:
    if isinstance(input, str):
        return open_file(input)
    elif isinstance(input, InputStream):
        return input
    else:
        raise TypeError("Invalid type: " + type(input))

logging.debug("Configure JVM-specific IO extensions")



import logging
import jpype
import jpype.imports

from it.unibo.tuprolog.core.operators import Operator
from it.unibo.tuprolog.core.operators import OperatorSet
from it.unibo.tuprolog.core.operators import Specifier

from tuprolog.pyutils import iterable_or_varargs
from tuprolog.jvmutils import jiterable, jmap
from tuprolog.core import Atom
from tuprolog.core import Integer
from tuprolog.core import Struct

from functools import singledispatch

logging.debug("Loaded JVM classes from it.unibo.tuprolog.core.operators.*")


@singledispatch
def operator(functor: str, specifier: Specifier, priority: int) -> Operator:
    return Operator(functor, specifier, priority)

@operator.register
def _(priority: Integer, specifier: Atom, functor: Atom) -> Operator:
    return Operator.fromTerms(priority, specifier, functor)

@operator.register
def _(term: Struct) -> Operator:
    return Operator.fromTerm(term)

def operator_set(*operators) -> OperatorSet:
    return iterable_or_varargs(operators, lambda os: OperatorSet(jiterable(os)))

def specifier(name: str) -> Specifier:
    return Specifier.valueOf(name)

EMPTY_OPERATORS: OperatorSet = OperatorSet.EMPTY

DEFAULT_OPERATORS: OperatorSet = OperatorSet.DEFAULT

STANDARD_OPERATORS: OperatorSet = OperatorSet.STANDARD

XF: Specifier = Specifier.XF

YF: Specifier = Specifier.YF

FX: Specifier = Specifier.FX

FY: Specifier = Specifier.FY

XFX: Specifier = Specifier.XFX

XFY: Specifier = Specifier.XFY

YFX: Specifier = Specifier.YFX

OperatorSet.__add__ = lambda this, other: this.plus(other)
OperatorSet.__sub__ = lambda this, other: this.minus(other)



import logging
import jpype
import jpype.imports

from _jpype import _JObject as JObjectClass

from java.util import NoSuchElementException 
from java.util import Map
from java.util import Iterator
from java.lang import Iterable
from java.lang import Object

from it.unibo.tuprolog.utils import PyUtils

from collections.abc import Iterable as PyIterable
from collections.abc import Iterator as PyIterator
from collections.abc import Mapping

from .jvmioutils import *


def jiterable(iterable: PyIterable) -> Iterable:
    assert isinstance(iterable, PyIterable)
    return Iterable@iterable


def jiterator(iterator: PyIterator) -> Iterator:
    assert isinstance(iterator, PyIterator)
    return Iterator@iterator


def jmap(dictionary: Mapping) -> Map:
    assert isinstance(dictionary, Mapping)
    return Map@dictionary


def _java_obj_repr(java_object: Object) -> str:
    return str(java_object.toString())


# replaces the default __repr__ implementation for java objects, making them use _java_obj_repr
JObjectClass.__repr__ = _java_obj_repr


@jpype.JImplementationFor("kotlin.sequences.Sequence")
class _KtSequence:
    def __jclass_init__(self):
        PyIterable.register(self)

    def __iter__(self):
        return PyUtils.iterable(self).iterator()


@jpype.JImplementationFor("java.util.stream.Stream")
class _JvmStream:
    def __jclass_init__(self):
        PyIterable.register(self)

    def __iter__(self):
        return self.iterator()


logging.debug("Configure JVM-specific extensions")



import logging
import jpype
import jpype.imports

# from tuprolog.pyutils import iterable_or_varargs
# from tuprolog.jvmutils import jiterable, jmap

from tuprolog.core import TermVisitor

logging.debug("Loaded compatibility layer for JVM subtypes of " + str(TermVisitor.class_.getName()))


@jpype.JImplements(TermVisitor)
class AbstractTermVisitor(object):

    @jpype.JOverride
    def defaultValue(self, term):
        raise NotImplementedError()

    @jpype.JOverride
    def visitTerm(self, term):
        return TermVisitor.DefaultImpls.visitTerm(self, term)

    @jpype.JOverride
    def visitVar(self, term):
        return TermVisitor.DefaultImpls.visitVar(self, term)

    @jpype.JOverride
    def visitConstant(self, term):
        return TermVisitor.DefaultImpls.visitConstant(self, term)

    @jpype.JOverride
    def visitStruct(self, term):
        return TermVisitor.DefaultImpls.visitStruct(self, term)

    @jpype.JOverride
    def visitCollection(self, term):
        return TermVisitor.DefaultImpls.visitCollection(self, term)

    @jpype.JOverride
    def visitAtom(self, term):
        return TermVisitor.DefaultImpls.visitAtom(self, term)

    @jpype.JOverride
    def visitTruth(self, term):
        return TermVisitor.DefaultImpls.visitTruth(self, term)

    @jpype.JOverride
    def visitNumeric(self, term):
        return TermVisitor.DefaultImpls.visitNumeric(self, term)

    @jpype.JOverride
    def visitInteger(self, term):
        return TermVisitor.DefaultImpls.visitInteger(self, term)

    @jpype.JOverride
    def visitReal(self, term):
        return TermVisitor.DefaultImpls.visitReal(self, term)

    @jpype.JOverride
    def visitBlock(self, term):
        return TermVisitor.DefaultImpls.visitBlock(self, term)

    @jpype.JOverride
    def visitEmpty(self, term):
        return TermVisitor.DefaultImpls.visitEmpty(self, term)

    @jpype.JOverride
    def visitEmptyBlock(self, term):
        return TermVisitor.DefaultImpls.visitEmptyBlock(self, term)

    @jpype.JOverride
    def visitList(self, term):
        return TermVisitor.DefaultImpls.visitList(self, term)

    @jpype.JOverride
    def visitCons(self, term):
        return TermVisitor.DefaultImpls.visitCons(self, term)

    @jpype.JOverride
    def visitEmptyList(self, term):
        return TermVisitor.DefaultImpls.visitEmptyList(self, term)

    @jpype.JOverride
    def visitTuple(self, term):
        return TermVisitor.DefaultImpls.visitTuple(self, term)

    @jpype.JOverride
    def visitIndicator(self, term):
        return TermVisitor.DefaultImpls.visitIndicator(self, term)

    @jpype.JOverride
    def visitClause(self, term):
        return TermVisitor.DefaultImpls.visitClause(self, term)

    @jpype.JOverride
    def visitRule(self, term):
        return TermVisitor.DefaultImpls.visitRule(self, term)

    @jpype.JOverride
    def visitFact(self, term):
        return TermVisitor.DefaultImpls.visitFact(self, term)

    @jpype.JOverride
    def visitDirective(self, term):
        return TermVisitor.DefaultImpls.visitDirective(self, term)





from setuptools import setup, find_packages
import pathlib
import subprocess

# current directory
here = pathlib.Path(__file__).parent.resolve()

version_file = here / 'VERSION'

print(f"Executing setup.py from {here}")

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')

def format_git_describe_version(version):
    if '-' in version:
        splitted = version.split('-')
        tag = splitted[0]
        index = f"dev{splitted[1]}" #{hex(int(splitted[1]))[2:]}"
        # commit = splitted[2] 
        # return f"{tag}.{index}+{commit}"
        return f"{tag}.{index}"
    else:
        return version

def get_version_from_git():
    try:
        process = subprocess.run(["git", "describe"], cwd=str(here), check=True, capture_output=True)
        version = process.stdout.decode('utf-8').strip()
        version = format_git_describe_version(version)
        with version_file.open('w') as f:
            f.write(version)
        return version
    except subprocess.CalledProcessError:
        # with version_file.open('r') as f:
        return version_file.read_text().strip()

# version = os.popen('git describe').read().strip()
version = get_version_from_git()

print(f"Detected version {version} from git describe")

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name='2ppy',  # Required
    version=version,
    description='Python-based implementation of tuProlog -- the open ecosystem for symbolic AI --, based on 2P-Kt',
    license='Apache 2.0 License',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tuProlog/2ppy', 
    author='Giovanni Ciatto', 
    author_email='giovanni.ciatto@unibo.it',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Interpreters',
        'Topic :: Software Development :: Libraries',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Prolog'
    ],
    keywords='prolog, symbolic ai, ecosystem, tuprolog, 2p, python',  # Optional
    # package_dir={'': 'src'},  # Optional
    packages=find_packages(),  # Required
    python_requires='>=3.6, <4',
    install_requires=['JPype1==1.3.0'],  # Optional
    zip_safe = False,
    platforms = "Independant",
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/tuProlog/2ppy/issues',
        # 'Funding': 'https://donate.pypi.org',
        # 'Say Thanks!': 'http://saythanks.io/to/example',
        'Source': 'https://github.com/tuProlog/2ppy',
    },
)
